--[[
	Author: Alternator (Massiner of Nathrezim)
	Date:	2020
	
	Scripts for Custom Actions

]]

local AddonName, AddonTable = ...;
local Engine = AddonTable.ButtonEngine;
local ScriptsCustom = Engine.ScriptsCustom;

local Core = Engine.Core;
local Util = Engine.Util;
local C = Engine.Constants;


--[[------------------------------------------------
	UpdateAction		
--------------------------------------------------]]
function ScriptsCustom.UpdateAction(BFButton)
	BFButton.ABW:SetAttribute("type", "macro");
	BFButton.ABW:SetAttribute("macrotext", BFButton.CustomAction.GetAction(BFButton.CustomData));
end


--[[------------------------------------------------
	Full Update
--------------------------------------------------]]
function ScriptsCustom.FullUpdate(BFButton)
	BFButton:UpdateIcon();
	BFButton:UpdateChecked();
	BFButton:UpdateUsable();
	BFButton:UpdateCooldown();
	BFButton:UpdateText();
	BFButton:UpdateGlow();
	
	if (GetMouseFocus() == BFButton.ABW) then
		BFButton.ABW:UpdateTooltip();
	end
end


--[[------------------------------------------------
	Icon
--------------------------------------------------]]
function ScriptsCustom.UpdateIcon(BFButton)
	BFButton.ABWIcon:SetTexture(BFButton.CustomAction.GetIcon(BFButton.CustomData));
end


--[[------------------------------------------------
	Checked
--------------------------------------------------]]
function ScriptsCustom.UpdateChecked(BFButton)
	BFButton.ABW:SetChecked(BFButton.CustomAction.IsChecked(BFButton.CustomData));
end


--[[------------------------------------------------
	Usable
--------------------------------------------------]]
function ScriptsCustom.UpdateUsable(BFButton)
	Core.UpdateButtonUsable(BFButton, BFButton.CustomAction.IsUsable(BFButton.CustomData));
end


--[[------------------------------------------------
	Cooldown
--------------------------------------------------]]
function ScriptsCustom.UpdateCooldown(BFButton)
	CooldownFrame_Set(BFButton.ABWCooldown, BFButton.CustomAction.GetCooldown(BFButton.CustomData));
end


--[[------------------------------------------------
	Text
--------------------------------------------------]]
function ScriptsCustom.UpdateText(BFButton)
	BFButton.ABWName:SetText(BFButton.CustomAction.GetText(BFButton.CustomData));
end


--[[------------------------------------------------
	Glow
--------------------------------------------------]]
function ScriptsCustom.UpdateGlow(BFButton)
	if (BFButton.CustomAction.IsGlowing(BFButton.CustomData)) then
		ActionButton_ShowOverlayGlow(BFButton.ABW);
	else
		ActionButton_HideOverlayGlow(BFButton.ABW);
	end
end


--[[------------------------------------------------
	Tooltip
--------------------------------------------------]]
function ScriptsCustom.UpdateTooltip(ABW)
	GameTooltip_SetDefaultAnchor(GameTooltip, ABW);
	ABW.BFButton.CustomAction.UpdateTooltip(ABW.BFButton.CustomData);
end


--[[------------------------------------------------
	Cursor
--------------------------------------------------]]
function ScriptsCustom.GetCursor(BFButton)
    return "custom", BFButton.CustomName, BFButton.CustomData, nil, BFButton.CustomAction.GetIcon(BFButton.CustomData);
end

